<?php

declare(strict_types=1);

namespace Flexlab_Google_Analytics_For_WooCommerce\Admin;

use Flexlab_Google_Analytics_For_WooCommerce\Traits\Forbidden;
use Flexlab_Google_Analytics_For_WooCommerce\Traits\Instance;

/**
 * Main class to handle the plugin's options saved in the database.
 */
class Options 
{
    use Forbidden;
    use Instance;

    /**
     * Option name for the API Secret.
     *
     * @var string
     */
    private static $api_secret_key = 'woocommerce_google_analytics__api_secret';

    /**
     * Option name for the Measurement ID.
     *
     * @var string
     */
    private static $measurement_id_key = 'woocommerce_google_analytics__measurement_id';

    /**
     * Option name for using a Google tag.
     *
     * @var string
     */
    private static $use_google_tag = 'woocommerce_google_analytics__use_google_tag';

    /**
     * Create options used in the plugin.
     *
     * @return void
     */
    public static function create_options() : void 
    {
        self::create_api_secret();
        self::create_measurement_id();
        self::create_use_google_tag();
    }

    /**
     * Delete options used in the plugin.
     *
     * @return void
     */
    public static function delete_options() : void 
    {
        self::delete_api_secret();
        self::delete_measurement_id();
        self::delete_use_google_tag();
    }

    /**
     * Get API Secret option name.
     *
     * @return string
     */
    public static function get_api_secret_key() : string 
    {
        return self::$api_secret_key;
    }

    /**
     * Get Measurement ID option name.
     *
     * @return string
     */
    public static function get_measurement_id_key() : string 
    {
        return self::$measurement_id_key;
    }

    /**
     * Get Use Google Tag option name.
     *
     * @return string
     */
    public static function get_use_google_tag_key() : string 
    {
        return self::$use_google_tag;
    }

    /**
     * Get API Secret option value.
     *
     * @return string
     */
    public static function get_api_secret() : string 
    {
        $value = get_option( self::$api_secret_key );

        return ( ! $value ) ? '' : $value;
    }

    /**
     * Get Measurement ID option value.
     *
     * @return string
     */
    public static function get_measurement_id() : string 
    {
        $value = get_option( self::$measurement_id_key );

        return ( ! $value ) ? '' : $value;
    }

    /**
     * Get Use Google Tag option value.
     *
     * @return string
     */
    public static function get_use_google_tag() : string 
    {
        $value = get_option( self::$use_google_tag );

        return ( ! $value ) ? 'no' : $value;
    }

    /**
     * Check if API Secret is valid.
     *
     * @return bool
     */
    public static function is_api_secret_valid() : bool 
    {
        $value = self::get_api_secret();

        return ( ! $value ) ? false : true;
    }

    /**
     * Check if Measurement ID is valid.
     *
     * @return bool
     */
    public static function is_measurement_id_valid() : bool 
    {
        $value = self::get_measurement_id();

        return ( ! $value ) ? false : true;
    }

    /**
     * Check if Use Google Tag is valid.
     *
     * @return bool
     */
    public static function is_use_google_tag_valid() : bool 
    {
        $value = self::get_use_google_tag();

        return ( 'yes' !== $value && 'no' !== $value ) ? false : true;
    }

    /**
     * Create API Secret option.
     *
     * @return void
     */
    private static function create_api_secret() : void 
    {
        add_option( self::$api_secret_key );
    }

    /**
     * Create Measurement ID option.
     *
     * @return void
     */
    private static function create_measurement_id() : void 
    {
        add_option( self::$measurement_id_key );
    }

    /**
     * Create Use Google Tag option.
     *
     * @return void
     */
    private static function create_use_google_tag() : void 
    {
        add_option( self::$use_google_tag, 'no' );
    }

    /**
     * Delete API Secret option.
     *
     * @return bool
     */
    private static function delete_api_secret() : bool 
    {
        return delete_option( self::$api_secret_key );
    }

    /**
     * Delete Measurement ID option.
     *
     * @return bool
     */
    private static function delete_measurement_id() : bool 
    {
        return delete_option( self::$measurement_id_key );
    }

    /**
     * Delete Use Google Tag option.
     *
     * @return bool
     */
    private static function delete_use_google_tag() : bool 
    {
        return delete_option( self::$use_google_tag );
    }
}
