<?php

declare(strict_types=1);

namespace Flexlab_Google_Analytics_For_WooCommerce\Admin;

use Flexlab_Google_Analytics_For_WooCommerce\Traits\Forbidden;
use Flexlab_Google_Analytics_For_WooCommerce\Traits\Instance;

/**
 * Main class to initiate the WooCommerce settings in admin.
 */
class Settings 
{
    use Forbidden;
    use Instance;

    /**
	 * Settings constructor.
	 */
    public function __construct()
    {
        $this->init_hooks();
    }

    /**
     * Add a new tab into the WooCommerce settings tabs.
     *
     * @param array $tabs
     * @return array
     */
    public function add_tab( $tabs ) : array 
    {
        $tabs['google_analytics'] = esc_html__( 'Google Analytics', 'flexlab-google-analytics-for-woocommerce' );

        return $tabs;
    }

    /**
     * Add fields using the WooCommerce admin settings class. 
     *
     * @return void
     */
    public function add_fields() : void 
    {
        woocommerce_admin_fields( $this->get_fields() );
    }

    /**
     * Save fields using the WooCommerce admin settings class. 
     *
     * @return void
     */
    public function save_fields() : void 
    {
        woocommerce_update_options( $this->get_fields() );
    }

    /**
     * Hooks into actions and filters.
     *
     * @return void
     */
    private function init_hooks() : void 
    {
        add_filter( 'woocommerce_settings_tabs_array', array( $this, 'add_tab' ), 50 );
        add_action( 'woocommerce_settings_tabs_google_analytics', array( $this, 'add_fields' ) );
        add_action( 'woocommerce_update_options_google_analytics', array( $this, 'save_fields' ) );
    }

    /**
     * Array fo fields in the format of WooCommerce admin settings class.
     *
     * @return array
     */
    private function get_fields() : array 
    {
        $fields = array(
            'section_begin' => array(
                'name' => esc_html__( 'Credentials', 'flexlab-google-analytics-for-woocommerce' ),
                'type' => 'title'
            ),
            'api_secret' => array(
                'id'   => Options::get_api_secret_key(),
                'name' => esc_html__( 'API Secret', 'flexlab-google-analytics-for-woocommerce' ),
                'type' => 'text',
                'desc' => sprintf(
                    esc_html__( 'To create a new secret, navigate in the Google Analytics UI to: %s', 'flexlab-google-analytics-for-woocommerce' ),
                    '<br /><strong>Admin > Data Streams > choose your stream > Measurement Protocol > Create</strong>'
                )
            ),
            'measurement_id' => array(
                'id'   => Options::get_measurement_id_key(),
                'name' => esc_html__( 'Measurement ID', 'flexlab-google-analytics-for-woocommerce' ),
                'type' => 'text',
                'desc' => sprintf(
                    esc_html__( 'The identifier for a Data Stream. Found in the Google Analytics UI under: %s', 'flexlab-google-analytics-for-woocommerce' ),
                    '<br /><strong>Admin > Data Streams > choose your stream > Measurement ID</strong>'
                )
            ),
            'use_google_tag' => array(
                'id'   => Options::get_use_google_tag_key(),
                'name' => esc_html__( 'Use Google tag', 'flexlab-google-analytics-for-woocommerce' ),
                'type' => 'checkbox',
                'desc' => sprintf(
                    esc_html__( 'By default the Google tag is not included in the source code. Check this if would like to include it. %s', 'flexlab-google-analytics-for-woocommerce' ),
                    '<br /><strong>Note:</strong> This option will add a independent Google tag within the <strong><code>&lt;head&gt;</code></strong> HTML tag.'
                )
            ),
            'section_end' => array(
                'type' => 'sectionend'
            )
        );

        return $fields;
    }
}
