<?php

declare(strict_types=1);

namespace Flexlab_Google_Analytics_For_WooCommerce;

/**
 * Autoloader class used to include/require files using namespaces.
 */
class Autoloader
{
    /**
	 * Namespace of the source code.
	 *
	 * @var string
	 */
    private $namespace = 'Flexlab_Google_Analytics_For_WooCommerce';

    /**
	 * Path to the includes directory.
	 *
	 * @var string
	 */
	private $includes_path = __DIR__ . DIRECTORY_SEPARATOR;

	/**
     * File prefix to be loaded.
     *
     * @var string
     */
    private $file_prefix = 'class-';

    /**
	 * Autoloader constructor.
	 */
	public function __construct() 
    {
		if ( function_exists( '__autoload' ) ) {
			spl_autoload_register( '__autoload' );
		}

		spl_autoload_register( array( $this, 'autoload' ) );
	}

    /**
	 * Autoload files from the given namespace.
	 *
	 * @param string $class
     * @return void
	 */
	private function autoload( $class ) : void 
    {
		if ( 0 !== strpos( $class, $this->namespace ) ) {
            return;
        }

        $class = str_replace( $this->namespace . '\\', '', $class );
        $parts = explode( '\\', $class );
        $class = array_pop( $parts );
        $class = strtolower( $class );

        $directories = $this->get_directory_from_class( $parts );
        $file        = $this->get_file_name_from_class( $class );

        $this->load_file( $this->includes_path . $directories . $file );
	}

    /**
	 * Build a directory path from the array.
	 *
	 * @param array $parts
	 * @return string
	 */
	private function get_directory_from_class( $directories ) : string 
    {
		if ( empty( $directories ) ) {
			return '';
		}

		$directories = implode( DIRECTORY_SEPARATOR, $directories );
		$directories = str_replace( '_', '-', $directories );
		$directories = strtolower( $directories ) . DIRECTORY_SEPARATOR;

		return $directories;
	}

    /**
	 * Take a class name and turn it into a file name.
	 *
	 * @param string $class
	 * @return string
	 */
	private function get_file_name_from_class( $class ) : string 
    {
		return $this->file_prefix . str_replace( '_', '-', $class ) . '.php';
	}

    /**
	 * Require the class file.
	 *
	 * @param string $path
	 * @return void
	 */
	private function load_file( $path ) : void 
    {
		if ( file_exists( $path ) && is_readable( $path ) ) {
			require_once $path;
		} else {
			wp_die( esc_html( "The file attempting to be loaded at $path does not exist." ) );
		}
	}
}

new Autoloader();
