<?php

declare(strict_types=1);

namespace Flexlab_Google_Analytics_For_WooCommerce;

use Flexlab_Google_Analytics_For_WooCommerce\Admin\Options;
use Flexlab_Google_Analytics_For_WooCommerce\Traits\Forbidden;
use Flexlab_Google_Analytics_For_WooCommerce\Traits\Instance;

/**
 * Main class to handle the Google Analytics API.
 * 
 * @link https://developers.google.com/analytics/devguides/collection/ga4
 * @link https://developers.google.com/analytics/devguides/collection/protocol/ga4
 */
class Google_Analytics 
{
    use Forbidden;
    use Instance;

    /**
     * Google Analytics API endpoint.
     *
     * @var string
     */
    private $url = 'https://www.google-analytics.com/mp/collect';

    /**
     * Secret API key generated in the Google Analytics UI.
     *
     * @var string
     */
    private $api_secret;

    /**
     * The measurement ID associated with a stream found in the Google Analytics UI.
     *
     * @var string
     */
    private $measurement_id;

    /**
     * A unique identifier for a client generated from Google Analytics saved in cookies.
     *
     * @var string
     */
    private $client_id;

    /**
	 * Google Analytics constructor.
	 */
    public function __construct()
    {
        $this->setup_options();
        $this->init_hooks();
    }

    /**
     * Set the client_id based on the client's cookie or create one and set it.
     *
     * @return void
     */
    public function setup_client_id() : void 
    {
        $measurement_id = $this->get_measurement_id();
        $measurement_id = str_replace( 'G-', '', $measurement_id );
        $client_id      = null;
        $cookies        = array_map( 'sanitize_text_field', $_COOKIE );

        if ( isset( $cookies[ '_ga_' . $measurement_id ] ) ) {
            $client_id = $cookies[ '_ga_' . $measurement_id ];
        } else {
            $client_id = $this->create_client_id();
        }

        $this->set_client_id( $client_id );
    }

    /**
     * Add the Google tag into the <head> tag.
     *
     * @return void
     */
    public function add_ga_code() : void 
    {
        $measurement_id = $this->get_measurement_id();

        ?>

        <!-- Flexlab Google Analytics for WooCommerce -->
        <!-- Google tag (gtag.js) -->
        <script async src="https://www.googletagmanager.com/gtag/js?id=<?php echo esc_attr( $measurement_id ); ?>"></script>
        <script>
            window.dataLayer = window.dataLayer || [];
            function gtag(){dataLayer.push(arguments);}
            gtag('js', new Date());

            gtag('config', '<?php echo esc_attr( $measurement_id ); ?>');
        </script>
        <!-- / Flexlab Google Analytics for WooCommerce -->
        
        <?php
    }

    /**
     * Build and send purchase event.
     *
     * @link https://developers.google.com/analytics/devguides/collection/protocol/ga4/reference/events#purchase
     * @param int $transaction_id
     * @param string $currency
     * @param float $value
     * @param float $tax
     * @param float $shipping
     * @param array $items
     * @param int $user_id
     * @return void
     */
    public function send_event_purchase( $order_id, $currency, $value, $tax, $shipping, $items, $user_id = 0 ) 
    {
        $order_params = array(
            'transaction_id' => $order_id,
            'tax'            => $tax,
            'shipping'       => $shipping
        );

        $event   = $this->build_event_for_api( 'purchase', $currency, $value, $items, $order_params );
        $payload = $this->build_payload_for_api( $event, $user_id );

        $this->send_request_to_api( $payload );
    }

    /**
     * Hooks into actions and filters.
     *
     * @return void
     */
    private function init_hooks() : void 
    {
        add_action( 'wp_loaded', array( $this, 'setup_client_id' ) );

        if ( 'yes' === Options::get_use_google_tag() ) {
            add_action( 'wp_head', array( $this, 'add_ga_code' ) );
        }
    }

    /**
     * Get and set options saved in the database.
     *
     * @return void
     */
    private function setup_options() : void 
    {
        $this->set_api_secret( Options::get_api_secret() );
        $this->set_measurement_id( Options::get_measurement_id() );
    }

    /**
     * Build the data into a item array format.
     *
     * @link https://developers.google.com/analytics/devguides/collection/protocol/ga4/reference?client_type=gtag#payload_post_body
     * @param string $name
     * @param string $currency
     * @param float $value
     * @param array $items
     * @param array $params
     * @return array
     */
    private function build_event_for_api( $name, $currency, $value, $items, $params = array() ) : array 
    {
        $event = array(
            'name' => $name,
            'params' => array(
                'currency' => $currency,
                'value' => $value,
                'items' => $items
            )
        );

        if ( ! empty( $params ) ) {
            $event['params'] = array_merge( $event['params'], $params );
        }

        return $event;
    }

    /**
     * Build the payload.
     *
     * @param array $events
     * @param int $user_id
     * @return array
     */
    private function build_payload_for_api( $events, $user_id = 0 ): array 
    {
        $payload = array(
            'client_id' => $this->get_client_id(),
        );

        if ( 0 !== $user_id ) {
            $payload['user_id'] = (string) $user_id;
        }

        $payload['events'] = array( $events );

        return $payload;
    }

    /**
     * Send the payload.
     *
     * @link https://developers.google.com/analytics/devguides/collection/protocol/ga4/reference?client_type=gtag#transport
     * @link https://developers.google.com/analytics/devguides/collection/protocol/ga4/reference?client_type=gtag#payload
     * @param array $payload
     * @return void
     */
    private function send_request_to_api( $payload ) : void 
    {
        $endpoint = $this->get_endpoint();
        $response = wp_remote_post( $endpoint, array(
            'headers' => array(
                'Content-Type' => 'application/json'
            ),
            'body' => wp_json_encode( $payload )
        ) );

        if ( 204 !== wp_remote_retrieve_response_code( $response ) ) {
            // TODO: maybe log something in future
        }
    }
    
    /**
     * Get the URL endpoint with params for authorization.
     *
     * @return string
     */
    private function get_endpoint() : string 
    {
        $url = $this->url . '?' . http_build_query( array(
            'api_secret' => $this->get_api_secret(),
            'measurement_id' => $this->get_measurement_id()
        ) );

        return $url;
    }

    /**
     * Getter for the api_secret.
     *
     * @return string
     */
    private function get_api_secret() : string 
    {
        return $this->api_secret;
    }

    /**
     * Setter for the api_secret.
     *
     * @param string $value
     * @return void
     */
    private function set_api_secret( $value ) : void 
    {
        $this->api_secret = $value;
    }
    
    /**
     * Getter for the measurement_id.
     *
     * @return string
     */
    private function get_measurement_id() : string 
    {
        return $this->measurement_id;
    }

    /**
     * Setter for the measurement_id.
     *
     * @param string $value
     * @return void
     */
    private function set_measurement_id( $value ) : void 
    {
        $this->measurement_id = $value;
    }

    /**
     * Creates a random string for client_id.
     *
     * @return string
     */
    private function create_client_id() : string 
    {
        $timestamp = time();
        
        if ( ! empty( $_SERVER['HTTP_CLIENT_IP'] ) ) {
            $ip = sanitize_text_field( $_SERVER['HTTP_CLIENT_IP'] );
        } elseif ( ! empty( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) {
            $ip = sanitize_text_field( $_SERVER['HTTP_X_FORWARDED_FOR'] );
        } else {
            $ip = sanitize_text_field( $_SERVER['REMOTE_ADDR'] );
        }

        $client_id = sanitize_text_field( $timestamp . '.' . $ip );

        return $client_id;
    }

    /**
     * Getter for the client_id.
     *
     * @return string
     */
    private function get_client_id() : string 
    {
        return $this->client_id;
    }

    /**
     * Setter for the client_id.
     *
     * @param string $value
     * @return void
     */
    private function set_client_id( $value ) : void 
    {
        $this->client_id = $value;
    }
}
