<?php

declare(strict_types=1);

namespace Flexlab_Google_Analytics_For_WooCommerce;

use Flexlab_Google_Analytics_For_WooCommerce\Admin\Options;
use Flexlab_Google_Analytics_For_WooCommerce\Admin\Settings;
use Flexlab_Google_Analytics_For_WooCommerce\Traits\Forbidden;
use Flexlab_Google_Analytics_For_WooCommerce\Traits\Instance;

/**
 * Main class to initiate the plugin.
 */
class Plugin 
{
    use Forbidden;
    use Instance;

    /**
	 * Plugin's version.
	 *
	 * @var string
	 */
	public $version = '1.0.0';

    /**
     * WooCommerce's plugin path;
     *
     * @var string
     */
    private $woocommerce_plugin_path = WP_PLUGIN_DIR . '/woocommerce/woocommerce.php';

    /**
     * WooCommerce's plugin basename;
     *
     * @var string
     */
    private $woocommerce_basename = 'woocommerce/woocommerce.php';

    /**
	 * Plugin constructor.
	 */
    public function __construct()
    {
        $this->define_constants();
        $this->init_hooks();
    }

    /**
     * On plugin activation create the options in the database.
     *
     * @return void
     */
    public function activate() : void 
    {
        Options::create_options();
    }

    /**
     * Plugin's initialization after all plugins have been loaded.
     *
     * @return void
     */
    public function init_plugin() 
    {
        $this->load_i18n();
        $this->init_dependencies();
    }

    /**
     * Show admin notice if WooCommerce is not installed/activated.
     *
     * @return void
     */
    public function show_admin_notice_if_woocommerce_is_missing() : void 
    {
        if ( ! $this->is_woocommerce_installed() || ! $this->is_woocommerce_activated() ) {
            $message = sprintf(
                esc_html__( 'WooCommerce Google Analytics requires %1$s to be installed and activated.', 'flexlab-google-analytics-for-woocommerce' ),
                '<a href="https://wordpress.org/plugins/woocommerce/">WooCommerce</a>'
            );

            if ( ! $this->is_woocommerce_installed() ) {
                $error_url = wp_nonce_url( admin_url( 'update.php?action=install-plugin&plugin=woocommerce' ), 'install-plugin_woocommerce' );
                $error_text = esc_html__( 'Install WooCommerce', 'flexlab-google-analytics-for-woocommerce' );
            } else {
                $error_url = wp_nonce_url( admin_url( 'plugins.php?action=activate&plugin=woocommerce/woocommerce.php' ), 'activate-plugin_woocommerce/woocommerce.php' );
                $error_text = esc_html__( 'Activate WooCommerce', 'flexlab-google-analytics-for-woocommerce' );
            }

            $message .= ' <a href="' . esc_url( $error_url ) . '">' . $error_text . '</a>.';

            ?>

            <div class="notice notice-error">
                <p><strong><?php echo esc_html__( 'WooCommerce Google Analytics' , 'flexlab-google-analytics-for-woocommerce' ); ?></strong></p>
                <p><?php echo wp_kses( $message, array( 'a' => array( 'href'  => array() ) ) ); ?></p>
            </div>

            <?php
        }
    }

    /**
     * Show admin notice if plugin options are invalid.
     *
     * @return void
     */
    public function show_admin_notice_if_api_secret_is_invalid() : void 
    {
        if ( $this->is_woocommerce_installed() && $this->is_woocommerce_activated() ) {
            if ( ! Options::is_api_secret_valid() || ! Options::is_measurement_id_valid() ) {
                $messages = array();

                if ( ! Options::is_api_secret_valid() ) {
                    $messages['api_secret']['text'] = esc_html__( 'Requires %1$s to work with Google Analytics API.', 'flexlab-google-analytics-for-woocommerce' );
                    $messages['api_secret']['term'] = '<strong>API Secret</strong>';
                }

                if ( ! Options::is_measurement_id_valid() ) {
                    $messages['measurement_id']['text'] = esc_html__( 'Requires %1$s to work with Google Analytics API.', 'flexlab-google-analytics-for-woocommerce' );
                    $messages['measurement_id']['term'] = '<strong>Measurement ID</strong>';
                }

                ?>

                <div class="notice notice-error">
                    <p><strong><?php echo esc_html__( 'WooCommerce Google Analytics' , 'flexlab-google-analytics-for-woocommerce' ); ?></strong></p>
                    <p>
                        <?php foreach ( $messages as $message ) : ?>
                            <?php echo wp_kses( sprintf( $message['text'], $message['term'] ), array( 'strong' => array() ) ); ?><br />
                        <?php endforeach; ?>
                    </p>
                    <p><a href="<?php echo esc_url( admin_url( 'admin.php?page=wc-settings&tab=google_analytics' ) ); ?>" aria-label="<?php echo esc_attr__( 'View WooCommerce Google Analytics settings', 'flexlab-google-analytics-for-woocommerce' ); ?>"><?php echo esc_html__( 'Settings', 'flexlab-google-analytics-for-woocommerce' ); ?></a></p>
                </div>

                <?php
            }
        }
    }

    /**
     * Show action links on the plugin screen.
     *
     * @param array $actions
     * @return array
     */
	public function plugin_action_links( $actions ) : array 
    {
		$action_links = array(
            'settings' => '<a href="' . esc_url( admin_url( 'admin.php?page=wc-settings&tab=google_analytics' ) ) . '" aria-label="' . esc_attr__( 'View WooCommerce Google Analytics settings', 'flexlab-google-analytics-for-woocommerce' ) . '">' . esc_html__( 'Settings', 'flexlab-google-analytics-for-woocommerce' ) . '</a>'
        );

		return array_merge( $action_links, $actions );
	}

    /**
     * Define constants.
     *
     * @return void
     */
    private function define_constants() : void 
    {
        $this->define( 'WOOCOMMERCE_GA_BASENAME', plugin_basename( WOOCOMMERCE_GA_PLUGIN_FILE ) );
        $this->define( 'WOOCOMMERCE_GA_NAMESPACE', 'Flexlab_Google_Analytics_For_WooCommerce' );
        $this->define( 'WOOCOMMERCE_GA_VERSION', $this->version );
        $this->define( 'WOOCOMMERCE_GA_PLUGIN', plugin_dir_path( WOOCOMMERCE_GA_PLUGIN_FILE ) );
        $this->define( 'WOOCOMMERCE_GA_PLUGIN_URI', plugin_dir_url( WOOCOMMERCE_GA_PLUGIN_FILE ) );
        $this->define( 'WOOCOMMERCE_GA_INCLUDES', WOOCOMMERCE_GA_PLUGIN . '/includes' );
    }

    /**
     * Define constant if not already set.
     *
     * @param string $name
     * @param string $value
     * @return void
     */
    private function define( $name, $value ) : void 
    {
        if ( ! defined( $name ) ) {
            define( $name, $value );
        }
    }

    /**
     * Hooks into actions and filters.
     *
     * @return void
     */
    private function init_hooks() : void 
    {
        add_action( 'plugins_loaded', array( $this, 'init_plugin' ) );
        add_action( 'admin_notices', array( $this, 'show_admin_notice_if_woocommerce_is_missing' ) );
        add_action( 'admin_notices', array( $this, 'show_admin_notice_if_api_secret_is_invalid' ) );

        add_filter( 'plugin_action_links_' . WOOCOMMERCE_GA_BASENAME, array( $this, 'plugin_action_links' ) );

        register_activation_hook( WOOCOMMERCE_GA_PLUGIN_FILE, array( $this, 'activate' ) );
    }

    /**
     * Load the text domain and internationalization.
     *
     * @return void
     */
    private function load_i18n() : void 
    {
		load_plugin_textdomain( 'woocommerce-google-analytics', false, WOOCOMMERCE_GA_PLUGIN . '/languages/' );
	}

    /**
     * Check if WooCommerce is installed in the plugins directory.
     *
     * @return bool
     */
    private function is_woocommerce_installed() : bool 
    {
        $installed_plugins = get_plugins();

        return isset( $installed_plugins[ $this->woocommerce_basename ] );
    }

    /**
     * Check if WooCommerce is activated in the plugins list.
     *
     * @return bool
     */
    private function is_woocommerce_activated() : bool 
    {
        // Check if woocommerce is active in single site
        if ( ! in_array( $this->woocommerce_plugin_path, wp_get_active_and_valid_plugins() ) ) {
            return false;
        }

        // Fallback to make sure the class exists
        if ( ! class_exists( 'WooCommerce' ) ) {
            return false;
        }

        return true;
    }

    /**
     * Initiate dedicated classes used in admin and public.
     *
     * @return void
     */
    private function init_dependencies() : void 
    {
        if ( Options::is_api_secret_valid() ) {
            Google_Analytics::init();
            WooCommerce::init();
        }

        if ( is_admin() ) {
            Settings::init();
        }
	}
}
