<?php

declare(strict_types=1);

namespace Flexlab_Google_Analytics_For_WooCommerce;

use Flexlab_Google_Analytics_For_WooCommerce\Traits\Forbidden;
use Flexlab_Google_Analytics_For_WooCommerce\Traits\Instance;
use Flexlab_Google_Analytics_For_WooCommerce\Google_Analytics;

/**
 * Main class to handle WooCommerce hooks.
 */
class WooCommerce 
{
    use Forbidden;
    use Instance;

    /**
     * The Google Analytics instance to build out items for the API.
     *
     * @var object
     */
    private $google_analytics;

    /**
	 * Plugin constructor.
	 */
    public function __construct()
    {
        $this->google_analytics = Google_Analytics::get_instance();
        $this->init_hooks();
    }

    /**
     * Send event to Google Analytics by setting the order status to processing.
     *
     * @return void
     */
    public function checkout_order_processed( $order_id, $posted_data, $order ) : void 
    {
        if ( ! $order ) {
            return;
        }

        $user_id     = get_current_user_id();
        $currency    = $this->get_wc_currency();
        $value       = (float) $order->get_total( false );
        $tax         = (float) $order->get_total_tax( false );
        $shipping    = (float) $order->get_shipping_total( false );
        $order_items = $order->get_items();
        $indexes     = $this->build_items_indexes( $order_items );
        $items       = array();

        foreach ( $order_items as $item ) {
            $product  = $this->get_wc_product( $item->get_product_id() );
            $quantity = $item->get_quantity();
            $index    = $this->get_item_index_for_ga( $indexes, $product->get_id() );
            $items[]  = $this->build_item_for_ga( $product, $quantity, $index );
        }

        $this->google_analytics->send_event_purchase( $order_id, $currency, $value, $tax, $shipping, $items, $user_id );
    }

    /**
     * Hooks into actions and filters.
     *
     * @return void
     */
    private function init_hooks() : void 
    {
        add_action( 'woocommerce_checkout_order_processed', array( $this, 'checkout_order_processed' ), 10, 3 );
    }

    /**
     * Get the WooCommerce currency code.
     *
     * @return string
     */
    private function get_wc_currency() : string 
    {
        return get_woocommerce_currency();
    }

    /**
     * Get the WooCommerce product object by id.
     *
     * @param int $id
     * @return object
     */
    private function get_wc_product( $id ) : ?object 
    {
        $product = wc_get_product( $id );

        return ( ! $product ) ? null : $product;
    }

    /**
     * Get the WooCommerce product categories by id.
     *
     * @param int $id
     * @return array
     */
    private function get_wc_product_categories( $id ) : array 
    {
        $categories = get_the_terms( $id, 'product_cat' );

        return ( ! $categories ) ? array() : $categories ;
    }

    /**
     * Build the item array in format needed for Google Analytics. 
     *
     * @param object $product
     * @param int $quantity
     * @return array
     */
    private function build_item_for_ga( $product, $quantity, $index ) : array 
    {
        $item_id    = ( ! empty( $product->get_sku() ) ) ? : (string) $product->get_id();
        $categories = $this->build_item_categories_for_ga( $product->get_id() );
        $price      = (float) $product->get_price();
        $discount   = 0;

        if ( ! empty( $product->get_sale_price() ) ) {
            $regular_price = (float) $product->get_regular_price();
            $sale_price    = (float) $product->get_sale_price();
            $discount      = $regular_price - $sale_price;
        }

        $item = array(
            'item_id' => $item_id,
            'item_name' => $product->get_name(),
            // 'affiliation' => 'Google Merchandise Store',
            // 'coupon' => 'SUMMER_FUN',
            // 'currency' => 'USD',
            'discount' => $discount,
            'index' => $index,
            // 'item_brand' => 'Google',
            // 'item_category' => 'Apparel',
            // 'item_category2' => 'Adult',
            // 'item_category3' => 'Shirts',
            // 'item_category4' => 'Crew',
            // 'item_category5' => 'Short sleeve',
            // 'item_list_id' => 'related_products',
            // 'item_list_name' => 'Related Products',
            // 'item_variant' => 'green',
            // 'location_id' => 'ChIJIQBpAG2ahYAR_6128GcTUEo',
            'price' => $price,
            'quantity' => $quantity
        );

        $item = array_merge( $item, $categories );

        return $item;
    }

    /**
     * Build the categories array in item format needed for Google Analytics.
     *
     * @param string $product_id
     * @return array
     */
    private function build_item_categories_for_ga( $product_id ) : array 
    {
        $categories = $this->get_wc_product_categories( $product_id );

        if ( ! is_array( $categories ) || empty( $categories ) ) {
			return array();
		}

        $category_tree   = $this->get_category_tree( $categories );
        $categories_flat = $this->flatten_category_tree( $category_tree );
        $item_categories = array();

        foreach ( $categories_flat as $index => $category ) {
            if ( 0 !== $index ) {
                $key = 'item_category' . ( $index + 1 );
            } else {
                $key = 'item_category';
            }
            
            $item_categories[ $key ] = $category;
            $index++;
        }

        return $item_categories;
    }

    /**
     * Get the product category tree which preserves the hierarchical order.
     *
     * @param array $categories
     * @return array
     */
    private function get_category_tree( $categories ) : array 
    {
        $categories    = wp_list_sort( $categories, 'term_id' );
        $category_tree = array();

        foreach ( $categories as $category ) {
            if ( 0 === $category->parent ) {
                $category_tree[ $category->term_id ]['name'] = $category->name;
                continue;
            }

            $ancestors = array_reverse( get_ancestors( $category->term_id, 'product_cat', 'taxonomy' ) );
            $this->build_category_tree( $category_tree, $category, $ancestors );
        }

        return $category_tree;
    }

    /**
     * Build the category tree from the child upwards.
     *
     * @param array $category_tree
     * @param WP_Term $child_category
     * @param array $ancestors
     * @param int $key
     * @param int $depth
     * @return array
     */
    private function build_category_tree( &$category_tree, $child_category, $ancestors, $key = 0, $depth = 0 ) : array 
    {
        $count = count( $ancestors );

        if ( $key < $count && $depth < 200 ) {
            if ( ! array_key_exists( $ancestors[ $key ], $category_tree ) ) {
                $category_tree[ $ancestors[ $key ] ]['children'] = array();
            }

            if ( $ancestors[ $key ] === $child_category->parent ) {
                $category_tree[ $ancestors[ $key ] ]['children'][ $child_category->term_id ]['name'] = $child_category->name;
            }

            $this->build_category_tree( $category_tree[ $ancestors[ $key ] ]['children'], $child_category, $ancestors, $key + 1, $depth + 1 );
        }

        return $category_tree;
    }

    /**
     * Flat a category tree into a uni-dimensional array.
     *
     * @param array $categories
     * @return array
     */
    private function flatten_category_tree( $categories ) : array 
    {   
        $iterator      = new \RecursiveIteratorIterator( new \RecursiveArrayIterator( $categories ) );
        $flatten_array = array();

        foreach ( $iterator as $value ) {
            $flatten_array[] = $value;
        }

        return $flatten_array;
    }

    /**
     * Build array of indexes with product IDs in order.
     *
     * @param array $items
     * @return array
     */
    private function build_items_indexes( $items ) : array 
    {
        $indexes = array();

        foreach ( $items as $item ) {
            if ( is_object( $item ) && $item instanceof \WC_Order_Item_Product ) {
                $indexes[] = $item->get_product_id();
            } else {
                $indexes[] = $item['product_id'];
            }
        }

        return $indexes;
    }

    /**
     * Get item index in array needed for Google Analytics.
     *
     * @param array $items
     * @param int $product_id
     * @return int
     */
    private function get_item_index_for_ga( $items, $product_id ) : int 
    {
        if ( ! is_array( $items ) || empty( $items ) ) {
            return 0;
        }

        $index = 0;

        if ( in_array( $product_id, $items ) ) {
            $index = array_search( $product_id, $items, true );
        }

        return $index;
    }
}
